use strict;

# create files links_intra.txt, links_inter_chr1.txt etc, in the specified output_dir

if ($#ARGV != 2)
{
    print "Usage: $0 <input_file_intra> <input_file_inter> <output_dir>\n";
    print "In the input files, only the first 4 columns and col 7 are important: chr, locus, chr, locus, pvalue\n";
    exit;
}


my $input_file_intra = $ARGV[0];
my $input_file_inter = $ARGV[1];
my $outdir = $ARGV[2];

my $num_shades = 12;

my @shades = ();
my $i;
my @OUT = ();
my $chrom;

if (! -e $outdir)
{
    system ("mkdir -p $outdir");
}

# I couldn't make circos work with z values written in the links file, so I'm trying to write a separate link file for every shade
for ($i=0; $i < $num_shades; $i++)  # +1 is for > the last
{
    open ($OUT[$i], ">$outdir/links_intra_$i.txt") || die ("Cannot open file $outdir/links_intra$i.txt");
}

# first do the intra
#open (OUT, ">$outdir/links_intra.txt") || die ("Cannot open file $outdir/links_intra.txt");
print "Creating file $outdir/links_intra.txt\n";
my $line;
my $counter = 1;
my @pvalues = ();
for ($chrom = 1; $chrom <= 16; $chrom++)
{
    @pvalues = ();
    @shades = ();
    # first go through all the lines for this chromosome and store the pvalues in an array
    open (FILE, $input_file_intra) || die ("Cannot open $input_file_intra");
    while ($line = <FILE>)
    {            
        my @cols = split (/\s+/,$line);
        if ($cols[0] == $chrom && $cols[2] == $chrom)
        {
            my $l = 0;
            $l = - log($cols[6])  if ($cols[6] > 0);
            push (@pvalues, $l);
        }
    }
    close (FILE);
    
    # now sort the pvalues ascendingly
    my @sortedp = sort {$a <=> $b} @pvalues;
    my $nump = @sortedp."";     
    for (my $i=0; $i < $num_shades-1; $i++)     # the last shade is the remaining values
    {
        $shades[$i] = $sortedp[$nump/$num_shades*($i+1)];
    }
    
    print "Chr $chrom, shades are @shades\n";
    
    # now go again and write the file
    open (FILE, $input_file_intra) || die ("Cannot open $input_file_intra");
    while ($line = <FILE>)
    {    
        # first go through all the lines for this chromosome and store the pvalues in an array
        my @cols = split (/\s+/,$line);
        if ($cols[0] == $chrom && $cols[2] == $chrom)
        {
            my $pvalue = $cols[6];
            my ($extra, $fileindex) = get_color($pvalue);
            
            print { $OUT[$fileindex] } "segdup$counter hs$cols[0] $cols[1] $cols[1] $extra\n";
            print { $OUT[$fileindex] } "segdup$counter hs$cols[2] $cols[3] $cols[3] $extra\n";
            $counter++;
        }
    }
    close (FILE);    
}


for ($i=0; $i <= $#shades; $i++)
{
    close ($OUT[$i]);
}


# now do the inter, one by one
 

for ($chrom = 1; $chrom <= 16; $chrom++)
{
    print "Creating file $outdir/links_inter_chr$chrom.txt\n";
    for ($i=0; $i < $num_shades; $i++)  # +1 is for > the last
    {
        open ($OUT[$i], ">$outdir/links_inter_chr${chrom}_$i.txt") || die ("Cannot open file $outdir/links_inter_chr${chrom}_$i.txt");
    }   
    my $counter = 1;
    
    # I have to gather and sort the pvalues for every pair of chromosomes
    for (my $chrom2 = $chrom+1; $chrom2 <= 16; $chrom2++)
    {        
        # first collect the p-values
        open (FILE, $input_file_inter) || die ("Cannot open $input_file_inter");              
        @pvalues = ();
        @shades = ();    
        while ($line = <FILE>)
        {
            my @cols = split (/\s+/,$line);
            if (($cols[0] == $chrom && $cols[2] == $chrom2) || ($cols[0] == $chrom2 && $cols[2] == $chrom))
            {
                my $l = 0;
                $l = - log($cols[6])  if ($cols[6] > 0);
                push (@pvalues, $l);            
            }
        }    
        close (FILE);    
        my @sortedp = sort {$a <=> $b} @pvalues;
        my $nump = @sortedp."";     
        for (my $i=0; $i < $num_shades-1; $i++)     # the last shade is the remaining values
        {
            $shades[$i] = $sortedp[$nump/$num_shades*($i+1)];
        }       
        print "Chr $chrom-$chrom2, shades are @shades\n";
                                
        # now go again and write the file
        open (FILE, $input_file_inter) || die ("Cannot open $input_file_inter");        
        while ($line = <FILE>)
        {
            my @cols = split (/\s+/,$line);
            if ($cols[0] == $chrom && $cols[2] == $chrom2)
            {
            
                my $pvalue = $cols[6];
                my ($extra, $fileindex) = get_color($pvalue);
            
                print { $OUT[$fileindex] } "segdup$counter hs$cols[0] $cols[1] $cols[1] $extra\n";
                print { $OUT[$fileindex] } "segdup$counter hs$cols[2] $cols[3] $cols[3] $extra\n";
                $counter++;
            }
            elsif ($cols[0] == $chrom2 && $cols[2] == $chrom)
            {
                my $pvalue = $cols[6];
                my ($extra, $fileindex) = get_color($pvalue);            
                print { $OUT[$fileindex] } "segdup$counter hs$cols[2] $cols[3] $cols[3] $extra\n";
                print { $OUT[$fileindex] } "segdup$counter hs$cols[0] $cols[1] $cols[1] $extra\n";            
                $counter++; 
            }
        }    
        close (FILE);
    }        
    for ($i=0; $i < $num_shades; $i++)  # +1 is for > the last
    {
        close ($OUT[$i]);
    }       
}


##################################

sub get_color
{
    my ($pvalue) = @_;
    my $logp = 0;
    if ($pvalue > 0)
    {
        $logp = - log($pvalue);
    }
    
    #print "logp = $logp\n";
    if    ($logp < $shades[0])      {   return ("color=vvvvlgrey", 0); }
    elsif ($logp < $shades[1])      {   return ("color=vvvlgrey", 1);  }
    elsif ($logp < $shades[2])      {   return ("color=vvlgrey", 2);   }
    elsif ($logp < $shades[3])      {   return ("color=vlgrey", 3);    }
    elsif ($logp < $shades[4])      {   return ("color=lgrey", 4);     }
    elsif ($logp < $shades[5])      {   return ("color=grey", 5);      }
    elsif ($logp < $shades[6])      {   return ("color=dgrey", 6);     }
    elsif ($logp < $shades[7])      {   return ("color=vdgrey", 7);    }
    elsif ($logp < $shades[8])      {   return ("color=vvdgrey", 8);   }
    elsif ($logp < $shades[9])      {   return ("color=vvvdgrey", 9);  }
    elsif ($logp < $shades[10])     {   return ("color=vvvvdgrey", 10); }
    else                            {   return ("color=black", 11);     }
    
}






