use strict;
my $circos = "~/Software/circos-0.48/bin/circos";

if ($#ARGV != 5)
{
    print "Usage: $0 <input_dir_with_links> <intra|inter_all|inter_pairwise|inter_specific> <from_chrom> <to_chrom> <format:png|svg> <radius:500|1500>\n";
    # input_dir can be for example: overlap1RE1 | SU1unionSU2_fdr0.01 | rDNA_interactions
    exit;
}

my $final = $ARGV[0];
my $intera = $ARGV[1];
my $from = $ARGV[2];
my $to = $ARGV[3];
my $format = $ARGV[4];
my $radius = $ARGV[5];
#my $radius = 1500;  # this makes very big files
#if ($format eq "svg")  { $radius = 150;}

my $num_shades = 12;

my $dir = "figures/${final}_${format}_$radius";

if (! -e $dir)
{
    system ("mkdir -p $dir");
}


if ($intera eq "inter_pairwise")
{
    for (my $i=$from; $i <= $to; $i++)
    {
        for (my $j=$i+1; $j <= $to; $j++)
        {
            print "Doing chromosome $i $j $intera ..... ";
            my $config = "/tmp/circos_${final}_${intera}_${i}_$j.conf";
            make_config($config, $i, $j, $intera);
            `$circos -conf $config`;
            print " done.\n";
        }
    }
}
elsif ($intera eq "inter_specific")
{
    my $i=$from;
    my $j=$to;
    #for (my $i=$from; $i <= $to; $i++)
    {
        #my $j=12;
        print "Doing chromosome $i $j $intera ..... ";
        my $config = "/tmp/circos_${final}_${intera}_${i}_$j.conf";
        make_config($config, $i, $j, $intera);
        `$circos -conf $config`;
        print " done.\n";
    }
}
else
{
    for (my $i=$from; $i <= $to; $i++)
    {
        print "Doing chromosome $i $intera ..... ";
        my $config = "/tmp/circos_${final}_${intera}_$i.conf";
        make_config($config, $i, 0, $intera);
        `$circos -conf $config`;

        print " done.\n";
    }
}

##########################

sub make_config
{
    my ($config_file, $chrom, $chrom2, $intera) = @_;
    #print "Writing config $config_file\n";
    open (CONF, ">$config_file") || die ("Cannot open $config_file");
    my $chromosomes_text;
    my $ideofile;
    my $karyotype;
    my $filename;
    my $rules = "";
    if ($intera eq "intra")
    {
        $chromosomes_text = "chromosomes = hs$chrom";
        $ideofile = "ideogram_1chrom.conf";
        $karyotype = "circos_files/karyotype.yeast.bands.color.txt";
        $filename = "intra_chr$chrom.$format";
    }
    elsif ($intera eq "inter_all")
    {
        $chromosomes_text = "chromosomes = hs1;hs2;hs3;hs4;hs5;hs6;hs7;hs8;hs9;hs10;hs11;hs12;hs13;hs14;hs15;hs16";
        $ideofile = "ideogram_allchrom.conf";
        $intera = "inter_chr$chrom";
        $karyotype = "circos_files/karyotype.yeast.bands.color.txt";
        $filename = "inter_chr${chrom}_vs_all.$format";
        $rules = "<rules>\n# remap the color of the link to the second chromosome\n<rule>\nimportance = 100\ncondition  = 1\ncolor      = eval(\"chr\".substr(_CHR2_,2))\nflow       = continue\n</rule>\n</rules>\n";
        
    }
    elsif ($intera eq "inter_pairwise" || $intera eq "inter_specific")
    {   
        $chromosomes_text = "chromosomes = hs$chrom;hs$chrom2";
        $ideofile = "ideogram_allchrom.conf";
        $intera = "inter_chr$chrom";
        $karyotype = "circos_files/karyotype.yeast.bands.color.txt";
        $filename = "inter_chr${chrom}_vs_chr${chrom2}.$format";
    }
    
    print CONF <<EOF;
    
<colors>
<<include circos_files/colors.conf>>
</colors>

<fonts>
<<include circos_files/fonts.conf>>
</fonts>

<<include circos_files/$ideofile>>
<<include circos_files/ticks.conf>>

# specify the karyotype file here
# but reduce frequency of ticks when increasing the 
# number of ideograms
karyotype = $karyotype

<image>
dir = $dir
file  = $filename
24bit = yes
#$format = yes
#svg = yes
# radius of inscribed circle in image
radius         = ${radius}p
background     = white
#background     = transparent
# by default angle=0 is at 3 o'clock position
angle_offset   = -90
#angle_orientation = counterclockwise

auto_alpha_colors = yes
auto_alpha_steps  = 5
</image>

chromosomes_units           = 1000

# only draw the chromosomes specified
chromosomes_display_default = no
#chromosomes = hs1;hs2
$chromosomes_text



################################################################
#
# define highlights within <highlights> block
#
# note that the radial axis grid is drawn on top of the highlights

<highlights>
# the default value for z-depth and fill_color for all highlights
#z = 0
<highlight>
file       = circos_files/trna_highlights.txt
r0         = 1r
r1         = 1.03r
fill_color = blue
stroke_color = blue
stroke_thickness = 2
</highlight>

<highlight>
file       = circos_files/centromere_highlights.txt
r0         = 1r
r1         = 1.05r
fill_color = red
stroke_color = red
stroke_thickness = 30
</highlight>

<highlight>
file       = circos_files/telomere_highlights.txt
r0         = 1r
r1         = 1.03r
fill_color = dred
stroke_color = dred
stroke_thickness = 2
</highlight>

<highlight>
#file       = circos_files/HindIII_highlights.txt
file       = circos_files/HindIII_midfragments_highlights.txt
r0         = 0.96r
r1         = 0.99r
fill_color = green
stroke_color = green
stroke_thickness = 3
</highlight>

<highlight>
#file       = circos_files/HindIII_unmappable_highlights.txt
file       = circos_files/HindIII_midfragments_unmappable_highlights.txt
r0         = 0.96r
r1         = 0.99r
fill_color = black
stroke_color = black
stroke_thickness = 3
</highlight>

</highlights>


# Links (bezier curves or straight lines) are defined in <links> blocks.
# Each link data set is defined within a <link>.
# As with highlights, parameters defined
# in the root of <links> affect all data sets and are considered
# global settings. Individual parameters value can be refined by
# values defined within <link> blocks, or additionally on each
# data line within the input file.

<links>

#z      = 0
radius = 0.95r

bezier_radius = 0.25r
crest = 0.5
bezier_radius_purity = 0.25

EOF


#print "$intera\n";
#print "$ARGV[1]\n";
if ($ARGV[1] eq "intra" || $ARGV[1] eq "inter_pairwise" || $ARGV[1] eq "inter_specific")
{
    #print "In intera $intera\n";
    my $i = 0;
    for ($i=0; $i < $num_shades; $i++)
    {
        print CONF "<link segdup$i>\n";
        print CONF "z=$i\n";
        print CONF "show         = yes\n";
        print CONF "thickness    = 2\n";
        print CONF "file         = circos_files/$final/links_${intera}_${i}.txt\n";
        print CONF "record_limit = 200000\n";
        print CONF "</link>\n";
    }
}
else
{
    print CONF "<link segdup>\n";
    print CONF "show         = yes\n";
    print CONF "color        = vvdgrey\n";
    print CONF "thickness    = 2\n";
    print CONF "file         = circos_files/$final/links_${intera}.txt\n";
    print CONF "record_limit = 200000\n";

    print CONF "$rules\n"; 

    print CONF "</link>\n";
}

print CONF <<EOF;
</links>

anglestep       = 0.5
minslicestep    = 10
beziersamples   = 40
debug           = no
warnings        = no
imagemap        = no

units_ok = bupr
units_nounit = n

#debug_group = ticks    

EOF
    close (CONF);
}
